# pyright: reportConstantRedefinition=none
"""The acal commands module.

These commands are used in the following models:
SMU2461

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - acal.count
    - acal.lastrun.internaltemp
    - acal.lastrun.tempdiff
    - acal.lastrun.time
    - acal.run()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class AcalLastrun(BaseTSPCmd):
    """The ``acal.lastrun`` command tree.

    Properties and methods:
        - ``.internaltemp``: The ``acal.lastrun.internaltemp`` attribute.
        - ``.tempdiff``: The ``acal.lastrun.tempdiff`` attribute.
        - ``.time``: The ``acal.lastrun.time`` attribute.
    """

    @property
    def internaltemp(self) -> str:
        """Access the ``acal.lastrun.internaltemp`` attribute.

        Description:
            - This attribute returns the internal temperature of the instrument when autocalibration
              was run.

        Usage:
            - Accessing this property will send the ``print(acal.lastrun.internaltemp)`` query.

        TSP Syntax:
            ```
            - print(acal.lastrun.internaltemp)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".internaltemp"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.internaltemp)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.internaltemp`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def tempdiff(self) -> str:
        """Access the ``acal.lastrun.tempdiff`` attribute.

        Description:
            - This attribute returns the difference between the internal temperature and the
              temperature when autocalibration was last run.

        Usage:
            - Accessing this property will send the ``print(acal.lastrun.tempdiff)`` query.

        TSP Syntax:
            ```
            - print(acal.lastrun.tempdiff)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".tempdiff"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.tempdiff)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.tempdiff`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def time(self) -> str:
        """Access the ``acal.lastrun.time`` attribute.

        Description:
            - This attribute returns the date and time when autocalibration was last run.

        Usage:
            - Accessing this property will send the ``print(acal.lastrun.time)`` query.

        TSP Syntax:
            ```
            - print(acal.lastrun.time)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".time"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.time)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.time`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Acal(BaseTSPCmd):
    """The ``acal`` command tree.

    Properties and methods:
        - ``.count``: The ``acal.count`` attribute.
        - ``.lastrun``: The ``acal.lastrun`` command tree.
        - ``.run()``: The ``acal.run()`` function.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "acal") -> None:
        super().__init__(device, cmd_syntax)
        self._lastrun = AcalLastrun(device, f"{self._cmd_syntax}.lastrun")

    @property
    def count(self) -> str:
        """Access the ``acal.count`` attribute.

        Description:
            - This attribute returns the number of times autocalibration has been run.

        Usage:
            - Accessing this property will send the ``print(acal.count)`` query.

        TSP Syntax:
            ```
            - print(acal.count)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".count"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.count)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.count`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def lastrun(self) -> AcalLastrun:
        """Return the ``acal.lastrun`` command tree.

        Sub-properties and sub-methods:
            - ``.internaltemp``: The ``acal.lastrun.internaltemp`` attribute.
            - ``.tempdiff``: The ``acal.lastrun.tempdiff`` attribute.
            - ``.time``: The ``acal.lastrun.time`` attribute.
        """
        return self._lastrun

    def run(self) -> None:
        """Run the ``acal.run()`` function.

        Description:
            - This function immediately runs autocalibration and stores the constants.

        TSP Syntax:
            ```
            - acal.run()
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.run()"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.run()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The header commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HEADer {ON|OFF|<NR1>}
    - HEADer?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Header(SCPICmdWrite, SCPICmdRead):
    """The ``HEADer`` command.

    Description:
        - This command specifies the Response Header Enable State that causes the oscilloscope to
          either include or omit headers on query responses.

    Usage:
        - Using the ``.query()`` method will send the ``HEADer?`` query.
        - Using the ``.verify(value)`` method will send the ``HEADer?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HEADer value`` command.

    SCPI Syntax:
        ```
        - HEADer {ON|OFF|<NR1>}
        - HEADer?
        ```

    Info:
        - ``OFF`` sets the Response Header Enable State to false. This causes the oscilloscope to
          omit headers on query responses, so that only the argument is returned.
        - ``ON`` sets the Response Header Enable State to true. This causes the oscilloscope to
          include headers on applicable query responses. You can then use the query response as a
          command.
        - ``<NR1>`` = 0 sets the Response Header Enable State to false; any other value sets this
          state to true.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "HEADer") -> None:
        super().__init__(device, cmd_syntax)



"""The calibrate commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CALibrate:INTERNal
    - CALibrate:INTERNal:STARt
    - CALibrate:INTERNal:STATus?
    - CALibrate:PWRUpstatus?
    - CALibrate?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class CalibratePwrupstatus(SCPICmdRead):
    """The ``CALibrate:PWRUpstatus`` command.

    Description:
        - This query-only command returns the current status of the power-up calibration.

    Usage:
        - Using the ``.query()`` method will send the ``CALibrate:PWRUpstatus?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:PWRUpstatus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CALibrate:PWRUpstatus?
        ```
    """


class CalibrateInternalStatus(SCPICmdRead):
    """The ``CALibrate:INTERNal:STATus`` command.

    Description:
        - This query-only command returns the current status of the signal path calibration. Note:
          When running SPC through the remote interface, calibration status cannot be obtained until
          after the SPC completes. SPC takes approximately 15 minutes per channel which means a
          total of 2 hours on an 8-channel model. Any remote command that performs an action on the
          instrument is also disabled until the SPC is complete.

    Usage:
        - Using the ``.query()`` method will send the ``CALibrate:INTERNal:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate:INTERNal:STATus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CALibrate:INTERNal:STATus?
        ```
    """


class CalibrateInternalStart(SCPICmdWriteNoArguments):
    """The ``CALibrate:INTERNal:STARt`` command.

    Description:
        - This command (no query form) starts the signal path calibration (SPC) of the analog
          channels. This command is the same as the ``CALibrate:INTERNal`` command. You can use the
          ``CALibrate:INTERNal:STATus?`` query to return the current status of the signal path
          calibration of the instrument. Note: When running SPC through the remote interface,
          calibration status cannot be obtained until after the SPC completes. SPC takes
          approximately 15 minutes per channel which means a total of 2 hours on an 8-channel model.
          Any remote command that performs an action on the instrument is also disabled until the
          SPC is complete.

    Usage:
        - Using the ``.write()`` method will send the ``CALibrate:INTERNal:STARt`` command.

    SCPI Syntax:
        ```
        - CALibrate:INTERNal:STARt
        ```
    """


class CalibrateInternal(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``CALibrate:INTERNal`` command.

    Description:
        - This command (no query form) starts the signal path calibration (SPC) of the instrument.
          You can use the ``CALibrate:INTERNal:STATus?`` query to return the current status of the
          signal path calibration of the instrument. Note: When running SPC through the remote
          interface, calibration status cannot be obtained until after the SPC completes. SPC takes
          approximately 15 minutes per channel which means a total of 2 hours on an 8-channel model.
          Any remote command that performs an action on the instrument is also disabled until the
          SPC is complete.

    Usage:
        - Using the ``.write()`` method will send the ``CALibrate:INTERNal`` command.

    SCPI Syntax:
        ```
        - CALibrate:INTERNal
        ```

    Properties:
        - ``.start``: The ``CALibrate:INTERNal:STARt`` command.
        - ``.status``: The ``CALibrate:INTERNal:STATus`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._start = CalibrateInternalStart(device, f"{self._cmd_syntax}:STARt")
        self._status = CalibrateInternalStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def start(self) -> CalibrateInternalStart:
        """Return the ``CALibrate:INTERNal:STARt`` command.

        Description:
            - This command (no query form) starts the signal path calibration (SPC) of the analog
              channels. This command is the same as the ``CALibrate:INTERNal`` command. You can use
              the ``CALibrate:INTERNal:STATus?`` query to return the current status of the signal
              path calibration of the instrument. Note: When running SPC through the remote
              interface, calibration status cannot be obtained until after the SPC completes. SPC
              takes approximately 15 minutes per channel which means a total of 2 hours on an
              8-channel model. Any remote command that performs an action on the instrument is also
              disabled until the SPC is complete.

        Usage:
            - Using the ``.write()`` method will send the ``CALibrate:INTERNal:STARt`` command.

        SCPI Syntax:
            ```
            - CALibrate:INTERNal:STARt
            ```
        """
        return self._start

    @property
    def status(self) -> CalibrateInternalStatus:
        """Return the ``CALibrate:INTERNal:STATus`` command.

        Description:
            - This query-only command returns the current status of the signal path calibration.
              Note: When running SPC through the remote interface, calibration status cannot be
              obtained until after the SPC completes. SPC takes approximately 15 minutes per channel
              which means a total of 2 hours on an 8-channel model. Any remote command that performs
              an action on the instrument is also disabled until the SPC is complete.

        Usage:
            - Using the ``.query()`` method will send the ``CALibrate:INTERNal:STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:INTERNal:STATus?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CALibrate:INTERNal:STATus?
            ```
        """
        return self._status


class Calibrate(SCPICmdRead):
    """The ``CALibrate`` command.

    Description:
        - This query returns the status of signal path calibration. Note: When running SPC through
          the remote interface, calibration status cannot be obtained until after the SPC completes,
          which can take several minutes.

    Usage:
        - Using the ``.query()`` method will send the ``CALibrate?`` query.
        - Using the ``.verify(value)`` method will send the ``CALibrate?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - CALibrate?
        ```

    Properties:
        - ``.internal``: The ``CALibrate:INTERNal`` command.
        - ``.pwrupstatus``: The ``CALibrate:PWRUpstatus`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CALibrate") -> None:
        super().__init__(device, cmd_syntax)
        self._internal = CalibrateInternal(device, f"{self._cmd_syntax}:INTERNal")
        self._pwrupstatus = CalibratePwrupstatus(device, f"{self._cmd_syntax}:PWRUpstatus")

    @property
    def internal(self) -> CalibrateInternal:
        """Return the ``CALibrate:INTERNal`` command.

        Description:
            - This command (no query form) starts the signal path calibration (SPC) of the
              instrument. You can use the ``CALibrate:INTERNal:STATus?`` query to return the current
              status of the signal path calibration of the instrument. Note: When running SPC
              through the remote interface, calibration status cannot be obtained until after the
              SPC completes. SPC takes approximately 15 minutes per channel which means a total of 2
              hours on an 8-channel model. Any remote command that performs an action on the
              instrument is also disabled until the SPC is complete.

        Usage:
            - Using the ``.write()`` method will send the ``CALibrate:INTERNal`` command.

        SCPI Syntax:
            ```
            - CALibrate:INTERNal
            ```

        Sub-properties:
            - ``.start``: The ``CALibrate:INTERNal:STARt`` command.
            - ``.status``: The ``CALibrate:INTERNal:STATus`` command.
        """
        return self._internal

    @property
    def pwrupstatus(self) -> CalibratePwrupstatus:
        """Return the ``CALibrate:PWRUpstatus`` command.

        Description:
            - This query-only command returns the current status of the power-up calibration.

        Usage:
            - Using the ``.query()`` method will send the ``CALibrate:PWRUpstatus?`` query.
            - Using the ``.verify(value)`` method will send the ``CALibrate:PWRUpstatus?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - CALibrate:PWRUpstatus?
            ```
        """
        return self._pwrupstatus



"""TekScopePC device driver module."""

import logging
import warnings

from tm_devices.commands import TekScopePCMixin
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.scopes.tekscope.tekscope import AbstractTekScope
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813

_logger: logging.Logger = logging.getLogger(__name__)


@family_base_class
class TekScopePC(TekScopePCMixin, AbstractTekScope):  # pyright: ignore[reportIncompatibleVariableOverride]
    """TekScopePC device driver."""

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################

    @cached_property
    def total_channels(self) -> int:
        """Return the total number of channels (all types)."""
        # TekScopePC supports a maximum of 8 channels
        return 8

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _reboot(self) -> None:
        """Reboot the device."""
        msg = (
            f"Rebooting is not supported for the {self.__class__.__name__} driver, "
            f"{self.name_and_alias} will not be rebooted."
        )
        _logger.warning(msg)
        warnings.warn(msg, stacklevel=3)



"""Mixins that define abstract methods for functionality that device drivers must implement."""

from .analysis_object_mixins import (
    BusMixin,
    HistogramMixin,
    MathMixin,
    MeasurementsMixin,
    PlotMixin,
    PowerMixin,
    ReferenceMixin,
    SearchMixin,
)
from .base_afg_source_channel import BaseAFGSourceChannel
from .base_source_channel import BaseSourceChannel
from .channel_control_mixin import ChannelControlMixin
from .licensed_mixin import LicensedMixin
from .screen_capture_mixin import ScreenCaptureMixin
from .signal_generator_mixin import SignalGeneratorMixin
from .usb_drives_mixin import USBDrivesMixin

__all__ = [
    "BaseAFGSourceChannel",
    "BaseSourceChannel",
    "BusMixin",
    "ChannelControlMixin",
    "HistogramMixin",
    "LicensedMixin",
    "MathMixin",
    "MeasurementsMixin",
    "PlotMixin",
    "PowerMixin",
    "ReferenceMixin",
    "ScreenCaptureMixin",
    "SearchMixin",
    "SignalGeneratorMixin",
    "USBDrivesMixin",
]



"""The mainwindow commands module.

These commands are used in the following models:
MSO2, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MAINWindow:BADGe:BRINgtoview <Qstring>
    - MAINWindow:FONTSize <NR1>
    - MAINWindow:RRBDisplaystate {ON|OFF|1|0}
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MainwindowRrbdisplaystate(SCPICmdWrite):
    """The ``MAINWindow:RRBDisplaystate`` command.

    Description:
        - Sets the display state of the Results readout bar to ON (displayed) or OFF (not
          displayed).

    Usage:
        - Using the ``.write(value)`` method will send the ``MAINWindow:RRBDisplaystate value``
          command.

    SCPI Syntax:
        ```
        - MAINWindow:RRBDisplaystate {ON|OFF|1|0}
        ```

    Info:
        - ``1`` or ON enables Autoset to change vertical settings.
        - ``0`` or OFF disables Autoset from changing vertical settings.
    """


class MainwindowFontsize(SCPICmdWrite):
    """The ``MAINWindow:FONTSize`` command.

    Description:
        - Sets the font size for UI text elements. Font size range is 12 to 19 points.

    Usage:
        - Using the ``.write(value)`` method will send the ``MAINWindow:FONTSize value`` command.

    SCPI Syntax:
        ```
        - MAINWindow:FONTSize <NR1>
        ```

    Info:
        - ``<NR1>`` specifies the font size.
    """


class MainwindowBadgeBringtoview(SCPICmdWrite):
    """The ``MAINWindow:BADGe:BRINgtoview`` command.

    Description:
        - This command is used to automatically scroll the specified badge to make it visible. The
          input to the command is the badge title name.

    Usage:
        - Using the ``.write(value)`` method will send the ``MAINWindow:BADGe:BRINgtoview value``
          command.

    SCPI Syntax:
        ```
        - MAINWindow:BADGe:BRINgtoview <Qstring>
        ```

    Info:
        - ``<Qstring>`` specifies a badge title name to make the badge visible.
    """


class MainwindowBadge(SCPICmdRead):
    """The ``MAINWindow:BADGe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MAINWindow:BADGe?`` query.
        - Using the ``.verify(value)`` method will send the ``MAINWindow:BADGe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bringtoview``: The ``MAINWindow:BADGe:BRINgtoview`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bringtoview = MainwindowBadgeBringtoview(device, f"{self._cmd_syntax}:BRINgtoview")

    @property
    def bringtoview(self) -> MainwindowBadgeBringtoview:
        """Return the ``MAINWindow:BADGe:BRINgtoview`` command.

        Description:
            - This command is used to automatically scroll the specified badge to make it visible.
              The input to the command is the badge title name.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``MAINWindow:BADGe:BRINgtoview value`` command.

        SCPI Syntax:
            ```
            - MAINWindow:BADGe:BRINgtoview <Qstring>
            ```

        Info:
            - ``<Qstring>`` specifies a badge title name to make the badge visible.
        """
        return self._bringtoview


class Mainwindow(SCPICmdRead):
    """The ``MAINWindow`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MAINWindow?`` query.
        - Using the ``.verify(value)`` method will send the ``MAINWindow?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.badge``: The ``MAINWindow:BADGe`` command tree.
        - ``.fontsize``: The ``MAINWindow:FONTSize`` command.
        - ``.rrbdisplaystate``: The ``MAINWindow:RRBDisplaystate`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "MAINWindow"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._badge = MainwindowBadge(device, f"{self._cmd_syntax}:BADGe")
        self._fontsize = MainwindowFontsize(device, f"{self._cmd_syntax}:FONTSize")
        self._rrbdisplaystate = MainwindowRrbdisplaystate(
            device, f"{self._cmd_syntax}:RRBDisplaystate"
        )

    @property
    def badge(self) -> MainwindowBadge:
        """Return the ``MAINWindow:BADGe`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MAINWindow:BADGe?`` query.
            - Using the ``.verify(value)`` method will send the ``MAINWindow:BADGe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bringtoview``: The ``MAINWindow:BADGe:BRINgtoview`` command.
        """
        return self._badge

    @property
    def fontsize(self) -> MainwindowFontsize:
        """Return the ``MAINWindow:FONTSize`` command.

        Description:
            - Sets the font size for UI text elements. Font size range is 12 to 19 points.

        Usage:
            - Using the ``.write(value)`` method will send the ``MAINWindow:FONTSize value``
              command.

        SCPI Syntax:
            ```
            - MAINWindow:FONTSize <NR1>
            ```

        Info:
            - ``<NR1>`` specifies the font size.
        """
        return self._fontsize

    @property
    def rrbdisplaystate(self) -> MainwindowRrbdisplaystate:
        """Return the ``MAINWindow:RRBDisplaystate`` command.

        Description:
            - Sets the display state of the Results readout bar to ON (displayed) or OFF (not
              displayed).

        Usage:
            - Using the ``.write(value)`` method will send the ``MAINWindow:RRBDisplaystate value``
              command.

        SCPI Syntax:
            ```
            - MAINWindow:RRBDisplaystate {ON|OFF|1|0}
            ```

        Info:
            - ``1`` or ON enables Autoset to change vertical settings.
            - ``0`` or OFF disables Autoset from changing vertical settings.
        """
        return self._rrbdisplaystate



"""The awgcontrol commands module.

These commands are used in the following models:
AWG5200

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AWGControl:ARSettings {ON|OFF|1|0}
    - AWGControl:CLOCk:DRATe <NR1>
    - AWGControl:CLOCk:DRATe?
    - AWGControl:CLOCk:PHASe:ADJust <NR1>
    - AWGControl:CLOCk:PHASe:ADJust?
    - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
    - AWGControl:CLOCk:SOURce?
    - AWGControl:COMPile <filename>
    - AWGControl:CONFigure:CNUMber?
    - AWGControl:PJUMp:JSTRobe {ON|OFF|1|0}
    - AWGControl:PJUMp:JSTRobe?
    - AWGControl:PJUMp:SEDGe {FALLing|RISing}
    - AWGControl:PJUMp:SEDGe?
    - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
    - AWGControl:RMODe?
    - AWGControl:RSTate?
    - AWGControl:RUN:IMMediate
    - AWGControl:SNAMe?
    - AWGControl:SREStore <filepath>[,<msus>]
    - AWGControl:SSAVe <filepath>[,<msus>]
    - AWGControl:STOP:IMMediate
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AwgcontrolStopImmediate(SCPICmdWriteNoArguments):
    """The ``AWGControl:STOP:IMMediate`` command.

    Description:
        - This command stops the output of a waveform or a sequence.

    Usage:
        - Using the ``.write()`` method will send the ``AWGControl:STOP:IMMediate`` command.

    SCPI Syntax:
        ```
        - AWGControl:STOP:IMMediate
        ```
    """


class AwgcontrolStop(SCPICmdRead):
    """The ``AWGControl:STOP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:STOP?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:STOP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``AWGControl:STOP:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = AwgcontrolStopImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> AwgcontrolStopImmediate:
        """Return the ``AWGControl:STOP:IMMediate`` command.

        Description:
            - This command stops the output of a waveform or a sequence.

        Usage:
            - Using the ``.write()`` method will send the ``AWGControl:STOP:IMMediate`` command.

        SCPI Syntax:
            ```
            - AWGControl:STOP:IMMediate
            ```
        """
        return self._immediate


class AwgcontrolSsave(SCPICmdWrite):
    """The ``AWGControl:SSAVe`` command.

    Description:
        - This command saves the AWG's setup with waveforms.

    Usage:
        - Using the ``.write(value)`` method will send the ``AWGControl:SSAVe value`` command.

    SCPI Syntax:
        ```
        - AWGControl:SSAVe <filepath>[,<msus>]
        ```
    """


class AwgcontrolSrestore(SCPICmdWrite):
    """The ``AWGControl:SREStore`` command.

    Description:
        - This command opens a setup file into the AWG's setup memory.

    Usage:
        - Using the ``.write(value)`` method will send the ``AWGControl:SREStore value`` command.

    SCPI Syntax:
        ```
        - AWGControl:SREStore <filepath>[,<msus>]
        ```
    """


class AwgcontrolSname(SCPICmdRead):
    """The ``AWGControl:SNAMe`` command.

    Description:
        - This command returns the AWG's most recently saved setup location. The response contains
          the full path for the file, including the disk drive letter (msus or, mass storage unit
          specifier).

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:SNAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:SNAMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AWGControl:SNAMe?
        ```
    """


class AwgcontrolRunImmediate(SCPICmdWriteNoArguments):
    """The ``AWGControl:RUN:IMMediate`` command.

    Description:
        - This command initiates the output of a waveform or sequence. This is equivalent to pushing
          the play button on the front-panel or display. The AWG can be put in the run state only
          when waveforms or sequences are assigned to channels.

    Usage:
        - Using the ``.write()`` method will send the ``AWGControl:RUN:IMMediate`` command.

    SCPI Syntax:
        ```
        - AWGControl:RUN:IMMediate
        ```
    """


class AwgcontrolRun(SCPICmdRead):
    """The ``AWGControl:RUN`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:RUN?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RUN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``AWGControl:RUN:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = AwgcontrolRunImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> AwgcontrolRunImmediate:
        """Return the ``AWGControl:RUN:IMMediate`` command.

        Description:
            - This command initiates the output of a waveform or sequence. This is equivalent to
              pushing the play button on the front-panel or display. The AWG can be put in the run
              state only when waveforms or sequences are assigned to channels.

        Usage:
            - Using the ``.write()`` method will send the ``AWGControl:RUN:IMMediate`` command.

        SCPI Syntax:
            ```
            - AWGControl:RUN:IMMediate
            ```
        """
        return self._immediate


class AwgcontrolRstate(SCPICmdRead):
    """The ``AWGControl:RSTate`` command.

    Description:
        - This command returns the run state of the AWG.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:RSTate?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RSTate?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AWGControl:RSTate?
        ```
    """


class AwgcontrolRmode(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:RMODe`` command.

    Description:
        - This command sets or returns the run mode of the AWG for Channel 1.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:RMODe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:RMODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:RMODe value`` command.

    SCPI Syntax:
        ```
        - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
        - AWGControl:RMODe?
        ```

    Info:
        - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for a trigger event).
          TRIGgered sets the Run Mode to Triggered, waiting for a trigger event. One waveform play
          out cycle completes, then play out stops, waiting for the next trigger event. GATed sets
          the Run Mode to only playout a waveform while the trigger is enabled.
        - ``*RST`` sets this to CONT.
    """


class AwgcontrolPjumpSedge(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:PJUMp:SEDGe`` command.

    Description:
        - This command sets or returns the active Strobe Edge to use for Pattern Jump when Pattern
          Jump is enabled for Sequencing.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp:SEDGe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:SEDGe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:SEDGe value`` command.

    SCPI Syntax:
        ```
        - AWGControl:PJUMp:SEDGe {FALLing|RISing}
        - AWGControl:PJUMp:SEDGe?
        ```

    Info:
        - ``FALLing`` sets the falling edge of the strobe signal to the active edge. 256 input
          patterns are available.
        - ``RISing`` sets the rising edge of the strobe signal to the active edge. 256 input
          patterns are available.
    """


class AwgcontrolPjumpJstrobe(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:PJUMp:JSTRobe`` command.

    Description:
        - This command sets or returns if the pattern jump is set (enabled or disabled) to always
          occur on the strobe signal. With this setting disabled, the pattern jump requires a strobe
          signal and a pattern address change to initiate a jump. With this setting enabled, the
          pattern jump disregards the pattern address change condition, causing the jump to always
          occur on the strobe signal.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:JSTRobe value``
          command.

    SCPI Syntax:
        ```
        - AWGControl:PJUMp:JSTRobe {ON|OFF|1|0}
        - AWGControl:PJUMp:JSTRobe?
        ```
    """


class AwgcontrolPjump(SCPICmdRead):
    """The ``AWGControl:PJUMp`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:PJUMp?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.jstrobe``: The ``AWGControl:PJUMp:JSTRobe`` command.
        - ``.sedge``: The ``AWGControl:PJUMp:SEDGe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._jstrobe = AwgcontrolPjumpJstrobe(device, f"{self._cmd_syntax}:JSTRobe")
        self._sedge = AwgcontrolPjumpSedge(device, f"{self._cmd_syntax}:SEDGe")

    @property
    def jstrobe(self) -> AwgcontrolPjumpJstrobe:
        """Return the ``AWGControl:PJUMp:JSTRobe`` command.

        Description:
            - This command sets or returns if the pattern jump is set (enabled or disabled) to
              always occur on the strobe signal. With this setting disabled, the pattern jump
              requires a strobe signal and a pattern address change to initiate a jump. With this
              setting enabled, the pattern jump disregards the pattern address change condition,
              causing the jump to always occur on the strobe signal.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:JSTRobe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:JSTRobe value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:PJUMp:JSTRobe {ON|OFF|1|0}
            - AWGControl:PJUMp:JSTRobe?
            ```
        """
        return self._jstrobe

    @property
    def sedge(self) -> AwgcontrolPjumpSedge:
        """Return the ``AWGControl:PJUMp:SEDGe`` command.

        Description:
            - This command sets or returns the active Strobe Edge to use for Pattern Jump when
              Pattern Jump is enabled for Sequencing.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp:SEDGe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp:SEDGe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:PJUMp:SEDGe value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:PJUMp:SEDGe {FALLing|RISing}
            - AWGControl:PJUMp:SEDGe?
            ```

        Info:
            - ``FALLing`` sets the falling edge of the strobe signal to the active edge. 256 input
              patterns are available.
            - ``RISing`` sets the rising edge of the strobe signal to the active edge. 256 input
              patterns are available.
        """
        return self._sedge


class AwgcontrolConfigureCnumber(SCPICmdRead):
    """The ``AWGControl:CONFigure:CNUMber`` command.

    Description:
        - This command returns the number of channels available on the AWG.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CONFigure:CNUMber?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure:CNUMber?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AWGControl:CONFigure:CNUMber?
        ```
    """


class AwgcontrolConfigure(SCPICmdRead):
    """The ``AWGControl:CONFigure`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CONFigure?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.cnumber``: The ``AWGControl:CONFigure:CNUMber`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._cnumber = AwgcontrolConfigureCnumber(device, f"{self._cmd_syntax}:CNUMber")

    @property
    def cnumber(self) -> AwgcontrolConfigureCnumber:
        """Return the ``AWGControl:CONFigure:CNUMber`` command.

        Description:
            - This command returns the number of channels available on the AWG.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CONFigure:CNUMber?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure:CNUMber?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AWGControl:CONFigure:CNUMber?
            ```
        """
        return self._cnumber


class AwgcontrolCompile(SCPICmdWrite):
    """The ``AWGControl:COMPile`` command.

    Description:
        - This command compiles an equation file and imports the waveforms (created by the equation
          file) into the arbitrary waveform generator.

    Usage:
        - Using the ``.write(value)`` method will send the ``AWGControl:COMPile value`` command.

    SCPI Syntax:
        ```
        - AWGControl:COMPile <filename>
        ```
    """


class AwgcontrolClockSource(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:SOURce`` command.

    Description:
        - This command sets or returns the source of the clock.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:SOURce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:SOURce value``
          command.

    SCPI Syntax:
        ```
        - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
        - AWGControl:CLOCk:SOURce?
        ```

    Info:
        - ``INTernal`` - clock signal is generated internally and the reference frequency is derived
          by the internal oscillator.EFIXed - clock is generated internally and the reference
          frequency is derived from a fixed 10 MHz reference supplied at the Reference In
          connector.EVARiable - clock is generated internally and the reference frequency is derived
          from a variable reference supplied at the Reference In connector.EXTernal - clock signal
          supplied by the Clock In connector and the reference frequency is derived from the
          internal precision oscillator.
        - ``*RST`` sets this to INT.
    """


class AwgcontrolClockPhaseAdjust(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:PHASe:ADJust`` command.

    Description:
        - This command sets or returns the internal clock phase adjustment of the AWG.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust value``
          command.

    SCPI Syntax:
        ```
        - AWGControl:CLOCk:PHASe:ADJust <NR1>
        - AWGControl:CLOCk:PHASe:ADJust?
        ```
    """


class AwgcontrolClockPhase(SCPICmdRead):
    """The ``AWGControl:CLOCk:PHASe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.adjust``: The ``AWGControl:CLOCk:PHASe:ADJust`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._adjust = AwgcontrolClockPhaseAdjust(device, f"{self._cmd_syntax}:ADJust")

    @property
    def adjust(self) -> AwgcontrolClockPhaseAdjust:
        """Return the ``AWGControl:CLOCk:PHASe:ADJust`` command.

        Description:
            - This command sets or returns the internal clock phase adjustment of the AWG.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe:ADJust?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AWGControl:CLOCk:PHASe:ADJust value`` command.

        SCPI Syntax:
            ```
            - AWGControl:CLOCk:PHASe:ADJust <NR1>
            - AWGControl:CLOCk:PHASe:ADJust?
            ```
        """
        return self._adjust


class AwgcontrolClockDrate(SCPICmdWrite, SCPICmdRead):
    """The ``AWGControl:CLOCk:DRATe`` command.

    Description:
        - This command sets or returns the divider rate for the external clock.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk:DRATe?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:DRATe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:DRATe value`` command.

    SCPI Syntax:
        ```
        - AWGControl:CLOCk:DRATe <NR1>
        - AWGControl:CLOCk:DRATe?
        ```

    Info:
        - ``*RST`` sets this to 1.
    """


class AwgcontrolClock(SCPICmdRead):
    """The ``AWGControl:CLOCk`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl:CLOCk?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.phase``: The ``AWGControl:CLOCk:PHASe`` command tree.
        - ``.drate``: The ``AWGControl:CLOCk:DRATe`` command.
        - ``.source``: The ``AWGControl:CLOCk:SOURce`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._phase = AwgcontrolClockPhase(device, f"{self._cmd_syntax}:PHASe")
        self._drate = AwgcontrolClockDrate(device, f"{self._cmd_syntax}:DRATe")
        self._source = AwgcontrolClockSource(device, f"{self._cmd_syntax}:SOURce")

    @property
    def phase(self) -> AwgcontrolClockPhase:
        """Return the ``AWGControl:CLOCk:PHASe`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:PHASe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:PHASe?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.adjust``: The ``AWGControl:CLOCk:PHASe:ADJust`` command.
        """
        return self._phase

    @property
    def drate(self) -> AwgcontrolClockDrate:
        """Return the ``AWGControl:CLOCk:DRATe`` command.

        Description:
            - This command sets or returns the divider rate for the external clock.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:DRATe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:DRATe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:DRATe value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:CLOCk:DRATe <NR1>
            - AWGControl:CLOCk:DRATe?
            ```

        Info:
            - ``*RST`` sets this to 1.
        """
        return self._drate

    @property
    def source(self) -> AwgcontrolClockSource:
        """Return the ``AWGControl:CLOCk:SOURce`` command.

        Description:
            - This command sets or returns the source of the clock.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk:SOURce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:CLOCk:SOURce value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:CLOCk:SOURce {INTernal|EXTernal|EFIXed|EVARiable}
            - AWGControl:CLOCk:SOURce?
            ```

        Info:
            - ``INTernal`` - clock signal is generated internally and the reference frequency is
              derived by the internal oscillator.EFIXed - clock is generated internally and the
              reference frequency is derived from a fixed 10 MHz reference supplied at the Reference
              In connector.EVARiable - clock is generated internally and the reference frequency is
              derived from a variable reference supplied at the Reference In connector.EXTernal -
              clock signal supplied by the Clock In connector and the reference frequency is derived
              from the internal precision oscillator.
            - ``*RST`` sets this to INT.
        """
        return self._source


class AwgcontrolArsettings(SCPICmdWrite):
    """The ``AWGControl:ARSettings`` command.

    Description:
        - This command sets or returns the state (enabled or disabled) of whether or not to apply
          the recommended settings of waveforms and sequences when they are assigned to a channel.
          When enabled, the system attempts to use the waveform's recommended settings (sample rate,
          amplitude, and offset) when the waveform is assigned to a channel. This includes waveforms
          within sequence tracks assigned to a channel. Recommended settings are defined as part the
          waveform properties and sequence properties.

    Usage:
        - Using the ``.write(value)`` method will send the ``AWGControl:ARSettings value`` command.

    SCPI Syntax:
        ```
        - AWGControl:ARSettings {ON|OFF|1|0}
        ```
    """


#  pylint: disable=too-many-instance-attributes
class Awgcontrol(SCPICmdRead):
    """The ``AWGControl`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AWGControl?`` query.
        - Using the ``.verify(value)`` method will send the ``AWGControl?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.arsettings``: The ``AWGControl:ARSettings`` command.
        - ``.clock``: The ``AWGControl:CLOCk`` command tree.
        - ``.compile``: The ``AWGControl:COMPile`` command.
        - ``.configure``: The ``AWGControl:CONFigure`` command tree.
        - ``.pjump``: The ``AWGControl:PJUMp`` command tree.
        - ``.rmode``: The ``AWGControl:RMODe`` command.
        - ``.rstate``: The ``AWGControl:RSTate`` command.
        - ``.run``: The ``AWGControl:RUN`` command tree.
        - ``.sname``: The ``AWGControl:SNAMe`` command.
        - ``.srestore``: The ``AWGControl:SREStore`` command.
        - ``.ssave``: The ``AWGControl:SSAVe`` command.
        - ``.stop``: The ``AWGControl:STOP`` command tree.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "AWGControl"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._arsettings = AwgcontrolArsettings(device, f"{self._cmd_syntax}:ARSettings")
        self._compile = AwgcontrolCompile(device, f"{self._cmd_syntax}:COMPile")
        self._configure = AwgcontrolConfigure(device, f"{self._cmd_syntax}:CONFigure")
        self._pjump = AwgcontrolPjump(device, f"{self._cmd_syntax}:PJUMp")
        self._rmode = AwgcontrolRmode(device, f"{self._cmd_syntax}:RMODe")
        self._rstate = AwgcontrolRstate(device, f"{self._cmd_syntax}:RSTate")
        self._run = AwgcontrolRun(device, f"{self._cmd_syntax}:RUN")
        self._sname = AwgcontrolSname(device, f"{self._cmd_syntax}:SNAMe")
        self._srestore = AwgcontrolSrestore(device, f"{self._cmd_syntax}:SREStore")
        self._ssave = AwgcontrolSsave(device, f"{self._cmd_syntax}:SSAVe")
        self._stop = AwgcontrolStop(device, f"{self._cmd_syntax}:STOP")
        self._clock = AwgcontrolClock(device, f"{self._cmd_syntax}:CLOCk")

    @property
    def arsettings(self) -> AwgcontrolArsettings:
        """Return the ``AWGControl:ARSettings`` command.

        Description:
            - This command sets or returns the state (enabled or disabled) of whether or not to
              apply the recommended settings of waveforms and sequences when they are assigned to a
              channel. When enabled, the system attempts to use the waveform's recommended settings
              (sample rate, amplitude, and offset) when the waveform is assigned to a channel. This
              includes waveforms within sequence tracks assigned to a channel. Recommended settings
              are defined as part the waveform properties and sequence properties.

        Usage:
            - Using the ``.write(value)`` method will send the ``AWGControl:ARSettings value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:ARSettings {ON|OFF|1|0}
            ```
        """
        return self._arsettings

    @property
    def compile(self) -> AwgcontrolCompile:
        """Return the ``AWGControl:COMPile`` command.

        Description:
            - This command compiles an equation file and imports the waveforms (created by the
              equation file) into the arbitrary waveform generator.

        Usage:
            - Using the ``.write(value)`` method will send the ``AWGControl:COMPile value`` command.

        SCPI Syntax:
            ```
            - AWGControl:COMPile <filename>
            ```
        """
        return self._compile

    @property
    def configure(self) -> AwgcontrolConfigure:
        """Return the ``AWGControl:CONFigure`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CONFigure?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CONFigure?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.cnumber``: The ``AWGControl:CONFigure:CNUMber`` command.
        """
        return self._configure

    @property
    def pjump(self) -> AwgcontrolPjump:
        """Return the ``AWGControl:PJUMp`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:PJUMp?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:PJUMp?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.jstrobe``: The ``AWGControl:PJUMp:JSTRobe`` command.
            - ``.sedge``: The ``AWGControl:PJUMp:SEDGe`` command.
        """
        return self._pjump

    @property
    def rmode(self) -> AwgcontrolRmode:
        """Return the ``AWGControl:RMODe`` command.

        Description:
            - This command sets or returns the run mode of the AWG for Channel 1.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:RMODe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RMODe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AWGControl:RMODe value`` command.

        SCPI Syntax:
            ```
            - AWGControl:RMODe {CONTinuous|TRIGgered|GATed}
            - AWGControl:RMODe?
            ```

        Info:
            - ``CONTinuous`` sets the Run Mode to Continuous (not waiting for a trigger event).
              TRIGgered sets the Run Mode to Triggered, waiting for a trigger event. One waveform
              play out cycle completes, then play out stops, waiting for the next trigger event.
              GATed sets the Run Mode to only playout a waveform while the trigger is enabled.
            - ``*RST`` sets this to CONT.
        """
        return self._rmode

    @property
    def rstate(self) -> AwgcontrolRstate:
        """Return the ``AWGControl:RSTate`` command.

        Description:
            - This command returns the run state of the AWG.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:RSTate?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RSTate?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AWGControl:RSTate?
            ```
        """
        return self._rstate

    @property
    def run(self) -> AwgcontrolRun:
        """Return the ``AWGControl:RUN`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:RUN?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:RUN?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``AWGControl:RUN:IMMediate`` command.
        """
        return self._run

    @property
    def sname(self) -> AwgcontrolSname:
        """Return the ``AWGControl:SNAMe`` command.

        Description:
            - This command returns the AWG's most recently saved setup location. The response
              contains the full path for the file, including the disk drive letter (msus or, mass
              storage unit specifier).

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:SNAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:SNAMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AWGControl:SNAMe?
            ```
        """
        return self._sname

    @property
    def srestore(self) -> AwgcontrolSrestore:
        """Return the ``AWGControl:SREStore`` command.

        Description:
            - This command opens a setup file into the AWG's setup memory.

        Usage:
            - Using the ``.write(value)`` method will send the ``AWGControl:SREStore value``
              command.

        SCPI Syntax:
            ```
            - AWGControl:SREStore <filepath>[,<msus>]
            ```
        """
        return self._srestore

    @property
    def ssave(self) -> AwgcontrolSsave:
        """Return the ``AWGControl:SSAVe`` command.

        Description:
            - This command saves the AWG's setup with waveforms.

        Usage:
            - Using the ``.write(value)`` method will send the ``AWGControl:SSAVe value`` command.

        SCPI Syntax:
            ```
            - AWGControl:SSAVe <filepath>[,<msus>]
            ```
        """
        return self._ssave

    @property
    def stop(self) -> AwgcontrolStop:
        """Return the ``AWGControl:STOP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:STOP?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:STOP?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``AWGControl:STOP:IMMediate`` command.
        """
        return self._stop

    @property
    def clock(self) -> AwgcontrolClock:
        """Return the ``AWGControl:CLOCk`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AWGControl:CLOCk?`` query.
            - Using the ``.verify(value)`` method will send the ``AWGControl:CLOCk?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.phase``: The ``AWGControl:CLOCk:PHASe`` command tree.
            - ``.drate``: The ``AWGControl:CLOCk:DRATe`` command.
            - ``.source``: The ``AWGControl:CLOCk:SOURce`` command.
        """
        return self._clock



# pyright: reportConstantRedefinition=none
"""The serial commands module.

These commands are used in the following models:
SMU2601B_Pulse

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - serial.baud
    - serial.databits
    - serial.parity
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Serial(BaseTSPCmd):
    """The ``serial`` command tree.

    Properties and methods:
        - ``.baud``: The ``serial.baud`` attribute.
        - ``.databits``: The ``serial.databits`` attribute.
        - ``.parity``: The ``serial.parity`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "serial") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def baud(self) -> str:
        """Access the ``serial.baud`` attribute.

        Description:
            - This attribute configures the baud rate for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.baud)`` query.
            - Setting this property to a value will send the ``serial.baud = value`` command.

        TSP Syntax:
            ```
            - serial.baud = value
            - print(serial.baud)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".baud"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.baud)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.baud`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @baud.setter
    def baud(self, value: Union[str, float]) -> None:
        """Access the ``serial.baud`` attribute.

        Description:
            - This attribute configures the baud rate for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.baud)`` query.
            - Setting this property to a value will send the ``serial.baud = value`` command.

        TSP Syntax:
            ```
            - serial.baud = value
            - print(serial.baud)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".baud", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.baud = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.baud`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def databits(self) -> str:
        """Access the ``serial.databits`` attribute.

        Description:
            - This attribute configures character width (data bits) for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.databits)`` query.
            - Setting this property to a value will send the ``serial.databits = value`` command.

        TSP Syntax:
            ```
            - serial.databits = value
            - print(serial.databits)
            ```

        Info:
            - ``bits``, the an integer representing the character width (7 or 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".databits"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.databits)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.databits`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @databits.setter
    def databits(self, value: Union[str, float]) -> None:
        """Access the ``serial.databits`` attribute.

        Description:
            - This attribute configures character width (data bits) for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.databits)`` query.
            - Setting this property to a value will send the ``serial.databits = value`` command.

        TSP Syntax:
            ```
            - serial.databits = value
            - print(serial.databits)
            ```

        Info:
            - ``bits``, the an integer representing the character width (7 or 8).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".databits", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.databits = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.databits`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def parity(self) -> str:
        """Access the ``serial.parity`` attribute.

        Description:
            - This attribute configures parity for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.parity)`` query.
            - Setting this property to a value will send the ``serial.parity = value`` command.

        TSP Syntax:
            ```
            - serial.parity = value
            - print(serial.parity)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".parity"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.parity)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.parity`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @parity.setter
    def parity(self, value: Union[str, float]) -> None:
        """Access the ``serial.parity`` attribute.

        Description:
            - This attribute configures parity for the RS-232 port.

        Usage:
            - Accessing this property will send the ``print(serial.parity)`` query.
            - Setting this property to a value will send the ``serial.parity = value`` command.

        TSP Syntax:
            ```
            - serial.parity = value
            - print(serial.parity)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".parity", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.parity = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.parity`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The ref commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - REF:ADDNew <QString>
    - REF:DELete <QString>
    - REF:LIST?
    - REF:REF<x>:DESKew <NR3>
    - REF:REF<x>:LABel:COLor <QString>
    - REF:REF<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
    - REF:REF<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
    - REF:REF<x>:LABel:FONT:SIZE <NR1>
    - REF:REF<x>:LABel:FONT:TYPE <QString>
    - REF:REF<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
    - REF:REF<x>:LABel:NAMe <QString>
    - REF:REF<x>:LABel:NAMe?
    - REF:REF<x>:LABel:XPOS <NR1>
    - REF:REF<x>:LABel:XPOS?
    - REF:REF<x>:LABel:YPOS <NR1>
    - REF:REF<x>:LABel:YPOS?
    - REF:REF<x>:SOUrce <QString>
    - REF:REF<x>:SOUrce?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class RefRefItemSource(SCPICmdWrite, SCPICmdRead):
    """The ``REF:REF<x>:SOUrce`` command.

    Description:
        - This command sets or queries the filename used by the given reference.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:SOUrce?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:SOUrce value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:SOUrce <QString>
        - REF:REF<x>:SOUrce?
        ```

    Info:
        - ``<QString>`` is the reference file name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class RefRefItemLabelYpos(SCPICmdWrite, SCPICmdRead):
    """The ``REF:REF<x>:LABel:YPOS`` command.

    Description:
        - This command sets or queries the Y-position of the label (attached to the displayed
          waveform of the specified reference), relative to the baseline of the waveform. The
          reference waveform is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:YPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:YPOS?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:YPOS value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:YPOS <NR1>
        - REF:REF<x>:LABel:YPOS?
        ```

    Info:
        - ``<NR1>`` is the location where the waveform label for the selected reference is
          displayed, relative to the baseline of the waveform.
    """


class RefRefItemLabelXpos(SCPICmdWrite, SCPICmdRead):
    """The ``REF:REF<x>:LABel:XPOS`` command.

    Description:
        - This command sets or queries the X-position at which the label (attached to the displayed
          waveform of the specified reference) is displayed, relative to the left edge of the
          waveview. The reference waveform is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:XPOS?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:XPOS?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:XPOS value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:XPOS <NR1>
        - REF:REF<x>:LABel:XPOS?
        ```

    Info:
        - ``<NR1>`` is the location (control in divisions) where the waveform label for the selected
          reference is displayed, relative to the left edge of the screen.
    """


class RefRefItemLabelName(SCPICmdWrite, SCPICmdRead):
    """The ``REF:REF<x>:LABel:NAMe`` command.

    Description:
        - This command sets or queries the label of the specified reference. The reference waveform
          is specified by x.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:NAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:NAMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:NAMe value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:NAMe <QString>
        - REF:REF<x>:LABel:NAMe?
        ```

    Info:
        - ``<QString>`` is the character string that will be used for the reference waveform label
          name.
    """

    _WRAP_ARG_WITH_QUOTES = True


class RefRefItemLabelFontUnderline(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:FONT:UNDERline`` command.

    Description:
        - This command sets or queries the underline state of the specified reference label.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:UNDERline value``
          command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
        ```

    Info:
        - ``<NR1>`` = 0 disables underline font; any other value turns this feature on.
        - ``OFF`` disables underline font.
        - ``ON`` enables underline font.
    """


class RefRefItemLabelFontType(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:FONT:TYPE`` command.

    Description:
        - This command sets or queries the font type of the specified reference label, such as Arial
          or Times New Roman.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:TYPE value``
          command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:FONT:TYPE <QString>
        ```

    Info:
        - ``<QString>`` is the font type.
    """

    _WRAP_ARG_WITH_QUOTES = True


class RefRefItemLabelFontSize(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:FONT:SIZE`` command.

    Description:
        - This command sets or queries the font size of the specified reference label.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:SIZE value``
          command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:FONT:SIZE <NR1>
        ```

    Info:
        - ``<NR1>`` is the font size of the label.
    """


class RefRefItemLabelFontItalic(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:FONT:ITALic`` command.

    Description:
        - This command sets or queries the italic state of the specified reference label.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:ITALic value``
          command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
        ```

    Info:
        - ``<NR1>`` = 0 disables italic font; any other value turns this feature on.
        - ``OFF`` disables italic font.
        - ``ON`` enables italic font.
    """


class RefRefItemLabelFontBold(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:FONT:BOLD`` command.

    Description:
        - This command sets or queries the bold state of the specified reference label.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:BOLD value``
          command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
        ```

    Info:
        - ``<NR1>`` = 0 disables bold font; any other value turns this feature on.
        - ``OFF`` disables bold font.
        - ``ON`` enables bold font.
    """


class RefRefItemLabelFont(SCPICmdRead):
    """The ``REF:REF<x>:LABel:FONT`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:FONT?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:FONT?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bold``: The ``REF:REF<x>:LABel:FONT:BOLD`` command.
        - ``.italic``: The ``REF:REF<x>:LABel:FONT:ITALic`` command.
        - ``.size``: The ``REF:REF<x>:LABel:FONT:SIZE`` command.
        - ``.type``: The ``REF:REF<x>:LABel:FONT:TYPE`` command.
        - ``.underline``: The ``REF:REF<x>:LABel:FONT:UNDERline`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bold = RefRefItemLabelFontBold(device, f"{self._cmd_syntax}:BOLD")
        self._italic = RefRefItemLabelFontItalic(device, f"{self._cmd_syntax}:ITALic")
        self._size = RefRefItemLabelFontSize(device, f"{self._cmd_syntax}:SIZE")
        self._type = RefRefItemLabelFontType(device, f"{self._cmd_syntax}:TYPE")
        self._underline = RefRefItemLabelFontUnderline(device, f"{self._cmd_syntax}:UNDERline")

    @property
    def bold(self) -> RefRefItemLabelFontBold:
        """Return the ``REF:REF<x>:LABel:FONT:BOLD`` command.

        Description:
            - This command sets or queries the bold state of the specified reference label.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:BOLD value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:FONT:BOLD {ON|OFF|<NR1>}
            ```

        Info:
            - ``<NR1>`` = 0 disables bold font; any other value turns this feature on.
            - ``OFF`` disables bold font.
            - ``ON`` enables bold font.
        """
        return self._bold

    @property
    def italic(self) -> RefRefItemLabelFontItalic:
        """Return the ``REF:REF<x>:LABel:FONT:ITALic`` command.

        Description:
            - This command sets or queries the italic state of the specified reference label.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``REF:REF<x>:LABel:FONT:ITALic value`` command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:FONT:ITALic {ON|OFF|<NR1>}
            ```

        Info:
            - ``<NR1>`` = 0 disables italic font; any other value turns this feature on.
            - ``OFF`` disables italic font.
            - ``ON`` enables italic font.
        """
        return self._italic

    @property
    def size(self) -> RefRefItemLabelFontSize:
        """Return the ``REF:REF<x>:LABel:FONT:SIZE`` command.

        Description:
            - This command sets or queries the font size of the specified reference label.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:SIZE value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:FONT:SIZE <NR1>
            ```

        Info:
            - ``<NR1>`` is the font size of the label.
        """
        return self._size

    @property
    def type(self) -> RefRefItemLabelFontType:
        """Return the ``REF:REF<x>:LABel:FONT:TYPE`` command.

        Description:
            - This command sets or queries the font type of the specified reference label, such as
              Arial or Times New Roman.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:FONT:TYPE value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:FONT:TYPE <QString>
            ```

        Info:
            - ``<QString>`` is the font type.
        """
        return self._type

    @property
    def underline(self) -> RefRefItemLabelFontUnderline:
        """Return the ``REF:REF<x>:LABel:FONT:UNDERline`` command.

        Description:
            - This command sets or queries the underline state of the specified reference label.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``REF:REF<x>:LABel:FONT:UNDERline value`` command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:FONT:UNDERline {ON|OFF|<NR1>}
            ```

        Info:
            - ``<NR1>`` = 0 disables underline font; any other value turns this feature on.
            - ``OFF`` disables underline font.
            - ``ON`` enables underline font.
        """
        return self._underline


class RefRefItemLabelColor(SCPICmdWrite):
    """The ``REF:REF<x>:LABel:COLor`` command.

    Description:
        - This command sets or queries the color of the specified ref label.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:COLor value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:LABel:COLor <QString>
        ```

    Info:
        - ``<QString>`` is the label. To return the color to the default color, send an empty string
          as in this example: ``:REF:REF1:LABEL:COLOR`` ''.
    """

    _WRAP_ARG_WITH_QUOTES = True


class RefRefItemLabel(SCPICmdRead):
    """The ``REF:REF<x>:LABel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.color``: The ``REF:REF<x>:LABel:COLor`` command.
        - ``.font``: The ``REF:REF<x>:LABel:FONT`` command tree.
        - ``.name``: The ``REF:REF<x>:LABel:NAMe`` command.
        - ``.xpos``: The ``REF:REF<x>:LABel:XPOS`` command.
        - ``.ypos``: The ``REF:REF<x>:LABel:YPOS`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._color = RefRefItemLabelColor(device, f"{self._cmd_syntax}:COLor")
        self._font = RefRefItemLabelFont(device, f"{self._cmd_syntax}:FONT")
        self._name = RefRefItemLabelName(device, f"{self._cmd_syntax}:NAMe")
        self._xpos = RefRefItemLabelXpos(device, f"{self._cmd_syntax}:XPOS")
        self._ypos = RefRefItemLabelYpos(device, f"{self._cmd_syntax}:YPOS")

    @property
    def color(self) -> RefRefItemLabelColor:
        """Return the ``REF:REF<x>:LABel:COLor`` command.

        Description:
            - This command sets or queries the color of the specified ref label.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:COLor value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:COLor <QString>
            ```

        Info:
            - ``<QString>`` is the label. To return the color to the default color, send an empty
              string as in this example: ``:REF:REF1:LABEL:COLOR`` ''.
        """
        return self._color

    @property
    def font(self) -> RefRefItemLabelFont:
        """Return the ``REF:REF<x>:LABel:FONT`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:FONT?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:FONT?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bold``: The ``REF:REF<x>:LABel:FONT:BOLD`` command.
            - ``.italic``: The ``REF:REF<x>:LABel:FONT:ITALic`` command.
            - ``.size``: The ``REF:REF<x>:LABel:FONT:SIZE`` command.
            - ``.type``: The ``REF:REF<x>:LABel:FONT:TYPE`` command.
            - ``.underline``: The ``REF:REF<x>:LABel:FONT:UNDERline`` command.
        """
        return self._font

    @property
    def name(self) -> RefRefItemLabelName:
        """Return the ``REF:REF<x>:LABel:NAMe`` command.

        Description:
            - This command sets or queries the label of the specified reference. The reference
              waveform is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:NAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:NAMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:NAMe value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:NAMe <QString>
            - REF:REF<x>:LABel:NAMe?
            ```

        Info:
            - ``<QString>`` is the character string that will be used for the reference waveform
              label name.
        """
        return self._name

    @property
    def xpos(self) -> RefRefItemLabelXpos:
        """Return the ``REF:REF<x>:LABel:XPOS`` command.

        Description:
            - This command sets or queries the X-position at which the label (attached to the
              displayed waveform of the specified reference) is displayed, relative to the left edge
              of the waveview. The reference waveform is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:XPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:XPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:XPOS value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:XPOS <NR1>
            - REF:REF<x>:LABel:XPOS?
            ```

        Info:
            - ``<NR1>`` is the location (control in divisions) where the waveform label for the
              selected reference is displayed, relative to the left edge of the screen.
        """
        return self._xpos

    @property
    def ypos(self) -> RefRefItemLabelYpos:
        """Return the ``REF:REF<x>:LABel:YPOS`` command.

        Description:
            - This command sets or queries the Y-position of the label (attached to the displayed
              waveform of the specified reference), relative to the baseline of the waveform. The
              reference waveform is specified by x.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:LABel:YPOS?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel:YPOS?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:LABel:YPOS value``
              command.

        SCPI Syntax:
            ```
            - REF:REF<x>:LABel:YPOS <NR1>
            - REF:REF<x>:LABel:YPOS?
            ```

        Info:
            - ``<NR1>`` is the location where the waveform label for the selected reference is
              displayed, relative to the baseline of the waveform.
        """
        return self._ypos


class RefRefItemDeskew(SCPICmdWrite):
    """The ``REF:REF<x>:DESKew`` command.

    Description:
        - This command sets or queries the deskew value used for the specified reference.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:REF<x>:DESKew value`` command.

    SCPI Syntax:
        ```
        - REF:REF<x>:DESKew <NR3>
        ```

    Info:
        - ``<NR3>`` is the deskew value used for the specified reference.
    """


class RefRefItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``REF:REF<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``REF:REF<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:REF<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.deskew``: The ``REF:REF<x>:DESKew`` command.
        - ``.label``: The ``REF:REF<x>:LABel`` command tree.
        - ``.source``: The ``REF:REF<x>:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._deskew = RefRefItemDeskew(device, f"{self._cmd_syntax}:DESKew")
        self._label = RefRefItemLabel(device, f"{self._cmd_syntax}:LABel")
        self._source = RefRefItemSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def deskew(self) -> RefRefItemDeskew:
        """Return the ``REF:REF<x>:DESKew`` command.

        Description:
            - This command sets or queries the deskew value used for the specified reference.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:DESKew value`` command.

        SCPI Syntax:
            ```
            - REF:REF<x>:DESKew <NR3>
            ```

        Info:
            - ``<NR3>`` is the deskew value used for the specified reference.
        """
        return self._deskew

    @property
    def label(self) -> RefRefItemLabel:
        """Return the ``REF:REF<x>:LABel`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:LABel?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.color``: The ``REF:REF<x>:LABel:COLor`` command.
            - ``.font``: The ``REF:REF<x>:LABel:FONT`` command tree.
            - ``.name``: The ``REF:REF<x>:LABel:NAMe`` command.
            - ``.xpos``: The ``REF:REF<x>:LABel:XPOS`` command.
            - ``.ypos``: The ``REF:REF<x>:LABel:YPOS`` command.
        """
        return self._label

    @property
    def source(self) -> RefRefItemSource:
        """Return the ``REF:REF<x>:SOUrce`` command.

        Description:
            - This command sets or queries the filename used by the given reference.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>:SOUrce?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``REF:REF<x>:SOUrce value`` command.

        SCPI Syntax:
            ```
            - REF:REF<x>:SOUrce <QString>
            - REF:REF<x>:SOUrce?
            ```

        Info:
            - ``<QString>`` is the reference file name.
        """
        return self._source


class RefList(SCPICmdRead):
    """The ``REF:LIST`` command.

    Description:
        - This command returns a comma separated list of all currently defined references.

    Usage:
        - Using the ``.query()`` method will send the ``REF:LIST?`` query.
        - Using the ``.verify(value)`` method will send the ``REF:LIST?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - REF:LIST?
        ```
    """


class RefDelete(SCPICmdWrite):
    """The ``REF:DELete`` command.

    Description:
        - Deletes the specified reference. Argument is of the form 'REF<NR1>', where NR1 ≥ 1.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:DELete value`` command.

    SCPI Syntax:
        ```
        - REF:DELete <QString>
        ```

    Info:
        - ``<QString>`` is the specified reference. Argument is of the form 'REF<NR1>', where NR1 ≥
          1.
    """

    _WRAP_ARG_WITH_QUOTES = True


class RefAddnew(SCPICmdWrite):
    """The ``REF:ADDNew`` command.

    Description:
        - This command adds the specified reference. Argument is of the form 'REF<NR1> ', where NR1
          ≥ 1.

    Usage:
        - Using the ``.write(value)`` method will send the ``REF:ADDNew value`` command.

    SCPI Syntax:
        ```
        - REF:ADDNew <QString>
        ```

    Info:
        - ``<QString>`` is the specified reference. Argument is of the form 'REF<NR1> ', where NR1 ≥
          1.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Ref(SCPICmdRead):
    """The ``REF`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``REF?`` query.
        - Using the ``.verify(value)`` method will send the ``REF?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.addnew``: The ``REF:ADDNew`` command.
        - ``.delete``: The ``REF:DELete`` command.
        - ``.list``: The ``REF:LIST`` command.
        - ``.ref``: The ``REF:REF<x>`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "REF") -> None:
        super().__init__(device, cmd_syntax)
        self._addnew = RefAddnew(device, f"{self._cmd_syntax}:ADDNew")
        self._delete = RefDelete(device, f"{self._cmd_syntax}:DELete")
        self._list = RefList(device, f"{self._cmd_syntax}:LIST")
        self._ref: Dict[int, RefRefItem] = DefaultDictPassKeyToFactory(
            lambda x: RefRefItem(device, f"{self._cmd_syntax}:REF{x}")
        )

    @property
    def addnew(self) -> RefAddnew:
        """Return the ``REF:ADDNew`` command.

        Description:
            - This command adds the specified reference. Argument is of the form 'REF<NR1> ', where
              NR1 ≥ 1.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:ADDNew value`` command.

        SCPI Syntax:
            ```
            - REF:ADDNew <QString>
            ```

        Info:
            - ``<QString>`` is the specified reference. Argument is of the form 'REF<NR1> ', where
              NR1 ≥ 1.
        """
        return self._addnew

    @property
    def delete(self) -> RefDelete:
        """Return the ``REF:DELete`` command.

        Description:
            - Deletes the specified reference. Argument is of the form 'REF<NR1>', where NR1 ≥ 1.

        Usage:
            - Using the ``.write(value)`` method will send the ``REF:DELete value`` command.

        SCPI Syntax:
            ```
            - REF:DELete <QString>
            ```

        Info:
            - ``<QString>`` is the specified reference. Argument is of the form 'REF<NR1>', where
              NR1 ≥ 1.
        """
        return self._delete

    @property
    def list(self) -> RefList:
        """Return the ``REF:LIST`` command.

        Description:
            - This command returns a comma separated list of all currently defined references.

        Usage:
            - Using the ``.query()`` method will send the ``REF:LIST?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:LIST?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - REF:LIST?
            ```
        """
        return self._list

    @property
    def ref(self) -> Dict[int, RefRefItem]:
        """Return the ``REF:REF<x>`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``REF:REF<x>?`` query.
            - Using the ``.verify(value)`` method will send the ``REF:REF<x>?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.deskew``: The ``REF:REF<x>:DESKew`` command.
            - ``.label``: The ``REF:REF<x>:LABel`` command tree.
            - ``.source``: The ``REF:REF<x>:SOUrce`` command.
        """
        return self._ref



"""The status_and_error commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K,
DPO7KC, DSA70KC, DSA70KD, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4,
MSO4B, MSO4K, MSO4KB, MSO5, MSO5B, MSO5K, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *PSC {ON|OFF|<NR1>}
    - *PSC?
    - *PUD {<Block>|<QString>}
    - *PUD?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Pud(SCPICmdWrite, SCPICmdRead):
    """The ``*PUD`` command.

    Description:
        - This command sets or queries a string of Protected User Data. This data is protected by
          the PASSWord command. You can modify it only by first entering the correct password. This
          password is not necessary to query the data.

    Usage:
        - Using the ``.query()`` method will send the ``*PUD?`` query.
        - Using the ``.verify(value)`` method will send the ``*PUD?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*PUD value`` command.

    SCPI Syntax:
        ```
        - *PUD {<Block>|<QString>}
        - *PUD?
        ```

    Info:
        - ``<Block>`` is a block containing up to 100 characters.
        - ``<QString>`` is a string containing up to 100 characters.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*PUD") -> None:
        super().__init__(device, cmd_syntax)


class Psc(SCPICmdWrite, SCPICmdRead):
    """The ``*PSC`` command.

    Description:
        - This command sets and queries the power-on status flag that controls the automatic
          power-on handling of the DESER, SRER, and ESER registers. When ``*PSC`` is true, the DESER
          register is set to 255 and the SRER and ESER registers are set to 0 at power-on. When
          ``*PSC`` is false, the current values in the DESER, SRER, and ESER registers are preserved
          in nonvolatile memory when power is shut off and are restored at power-on.

    Usage:
        - Using the ``.query()`` method will send the ``*PSC?`` query.
        - Using the ``.verify(value)`` method will send the ``*PSC?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*PSC value`` command.

    SCPI Syntax:
        ```
        - *PSC {ON|OFF|<NR1>}
        - *PSC?
        ```

    Info:
        - ``<NR1>`` = 0 sets the power-on status clear flag to false, disables the power-on clear
          and allows the instrument to possibly assert SRQ after power-on; any other value sets the
          power-on status clear flag to true, enabling the power-on status clear and prevents any
          SRQ assertion after power on.
        - ``OFF`` sets the power-on status clear flag to false, disables the power-on clear and
          allows the instrument to possibly assert SRQ after power-on.
        - ``ON`` sets the power-on status clear flag to true, enabling the power-on status clear and
          prevents any SRQ assertion after power on.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*PSC") -> None:
        super().__init__(device, cmd_syntax)



"""A private mixin for common methods and attributes for Tektronix scopes."""

from abc import ABC
from typing import Tuple

from tm_devices.driver_mixins.device_control._abstract_device_visa_write_query_control import (
    _AbstractDeviceVISAWriteQueryControl,  # pyright: ignore[reportPrivateUsage]
)


class _TektronixPIScopeMixin(_AbstractDeviceVISAWriteQueryControl, ABC):  # pyright: ignore[reportUnusedClass]
    """A private mixin for common methods and attributes for Tektronix scopes.

    !!! important
        Any class that inherits this mixin must also inherit the
        [`PIControl`][tm_devices.driver_mixins.device_control.PIControl] mixin in order
        to have access to the methods required by this class.
    """

    @property
    def _no_error_string(self) -> str:
        """A string containing the expected error message when no error is present."""
        return '0,"No events to report - queue empty"'

    def _get_errors(self) -> Tuple[int, Tuple[str, ...]]:
        """Get the current errors from the device.

        !!! note
            This method will clear out the error queue after reading the current errors.

        Returns:
            A tuple containing the current error code alongside a tuple of the current error
                messages.
        """
        result = int(self.query("*ESR?").strip())
        allev_list = [
            x + ('"' if not x.endswith('"') else "") for x in self.query(":ALLev?").split('",')
        ]
        returned_errors = tuple(filter(None, allev_list))

        return result, returned_errors



"""The status_and_error commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *ESE <NR1>
    - *ESE?
    - *OPC
    - *OPC?
    - *RST
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Rst(SCPICmdWriteNoArguments):
    """The ``*RST`` command.

    Description:
        - This command (no query form) resets the instrument to the factory default settings. This
          command does the following: Recalls the default instrument setup. Clears the current
          ``*DDT`` command. Disables aliases (``:ALIAS:STATE 0``). Disables the user password (for
          the ``*PUD`` command). The ``*RST`` command does not change the following: The current
          working directory (: ``FILESystem:CWD`` command). The state of command headers (: HEADer
          command). The state of keyword and enumeration verbosity (: VERBose command). The Power-on
          Status Clear Flag ( ``*PSC`` command). The Event Status Enable Register ( ``*ESE``
          command). The Service Request Enable Register ( ``*SRE`` command). The Event Status Enable
          Register ( ``*ESE`` command). The Service Request Enable Register ( ``*SRE`` command). The
          Device Event Status Enable Register ( DESE command). The user password for protected user
          data (: PASSWord command). The content of protected user data ( ``*PUD`` command). The
          enabled state of the socket server (: ``SOCKETServer:ENAble`` command). The socket server
          port number (: ``SOCKETServer:PORT`` command). The socket server protocol (:
          ``SOCKETServer:PROTOCol`` command). The USBTMC port configuration (:
          ``USBDevice:CONFigure`` command). The destination reference waveform or file path for the
          : CURVe command (: ``DATa:DESTination`` command). The source waveform for the : CURVe? or
          : WAVFrm? queries (: ``DATa:SOUrce`` command). The source waveform for the : CURVe? or :
          WAVFrm? queries (: ``DATa:SOUrce`` command). The waveform data encoding for the : CURVe
          command or query or the : WAVFrm? query (: ``DATa:ENCdg`` command). The starting point for
          : CURVe? queries (: ``DATa:STARt`` command). The ending point for : CURVe? queries (:
          ``DATa:STOP`` command). All settings associated the : WFMInpre commands. All user settable
          settings associated with the WFMOutpre commands. ``*RST`` only resets the programmable
          interface settings, it does not change the user interface settings.

    Usage:
        - Using the ``.write()`` method will send the ``*RST`` command.

    SCPI Syntax:
        ```
        - *RST
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*RST") -> None:
        super().__init__(device, cmd_syntax)


class Opc(SCPICmdWriteNoArguments, SCPICmdRead):
    """The ``*OPC`` command.

    Description:
        - This command generates the operation complete message in the Standard Event Status
          Register (SESR) when all pending commands that generate an OPC message are complete. The
          ``*OPC?`` query places the ASCII character '1' into the output queue when all such OPC
          commands are complete. The ``*OPC?`` response is not available to read until all pending
          operations finish. For a complete discussion of the use of these registers and the output
          queue, see Registers and Queues. The ``*OPC`` command allows you to synchronize the
          operation of the instrument with your application program. For more information, see
          Synchronization Methods. Refer to the Oscilloscope operations that can generate OPC table
          for a list of commands that generate an OPC message. (See Table 3-3.)

    Usage:
        - Using the ``.query()`` method will send the ``*OPC?`` query.
        - Using the ``.verify(value)`` method will send the ``*OPC?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write()`` method will send the ``*OPC`` command.

    SCPI Syntax:
        ```
        - *OPC
        - *OPC?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*OPC") -> None:
        super().__init__(device, cmd_syntax)


class Ese(SCPICmdWrite, SCPICmdRead):
    """The ``*ESE`` command.

    Description:
        - This command sets and queries the bits in the Event Status Enable Register (ESER). The
          ESER prevents events from being reported to the Status Byte Register (STB). For a more
          detailed discussion of the use of these registers, see Registers. Note: Setting the DESER
          and the ESER to the same values allows only those codes to be entered into the Event Queue
          and summarized on the ESB bit (bit 5) of the Status Byte Register. Use the DESE command to
          set the DESER.

    Usage:
        - Using the ``.query()`` method will send the ``*ESE?`` query.
        - Using the ``.verify(value)`` method will send the ``*ESE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*ESE value`` command.

    SCPI Syntax:
        ```
        - *ESE <NR1>
        - *ESE?
        ```

    Info:
        - ``<NR1>`` specifies the binary bits of the ESER according to this value, which ranges from
          0 through 255. The power-on default for the ESER is 0 if ``*PSC`` is 1. If ``*PSC`` is 0,
          the ESER maintains the previous power cycle value through the current power cycle.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*ESE") -> None:
        super().__init__(device, cmd_syntax)



"""SMU Model 26xxA device driver module."""

from abc import ABC

from tm_devices.drivers.source_measure_units.smu26xx.smu26xx import SMU26xx


class SMU26xxA(SMU26xx, ABC):
    """Base SMU26xxA device driver."""



"""The acquire commands module.

These commands are used in the following models:
DPO7AX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - ACQuire:FASTAVerage:LIMit <NR1>
    - ACQuire:FASTAVerage:LIMit?
    - ACQuire:FASTAVerage:STATE {ON|OFF|1|0}
    - ACQuire:FASTAVerage:STATE?
    - ACQuire:FASTAVerage:STOPafter <NR1>
    - ACQuire:FASTAVerage:STOPafter?
    - ACQuire:MAXSamplerate?
    - ACQuire:MODe {SAMple|PEAKdetect|AVErage|ENVelope}
    - ACQuire:MODe?
    - ACQuire:NUMACq?
    - ACQuire:NUMAVg <NR1>
    - ACQuire:NUMAVg?
    - ACQuire:NUMFRAMESACQuired?
    - ACQuire:SEQuence:CURrent?
    - ACQuire:SEQuence:MODe NUMACQs
    - ACQuire:SEQuence:NUMSEQuence <NR1>
    - ACQuire:SEQuence:NUMSEQuence?
    - ACQuire:STATE {<NR1>|OFF|ON|RUN|STOP}
    - ACQuire:STATE?
    - ACQuire:STOPAfter {RUNSTop|SEQuence}
    - ACQuire:STOPAfter?
    - ACQuire?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AcquireStopafter(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:STOPAfter`` command.

    Description:
        - This command sets or queries whether the instrument continually acquires acquisitions or
          acquires a single sequence. Pressing SINGLE on the front panel button is equivalent to
          sending these commands: ``ACQUIRE:STOPAFTER SEQUENCE`` and ``ACQUIRE:STATE 1``.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:STOPAfter?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:STOPAfter?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:STOPAfter value`` command.

    SCPI Syntax:
        ```
        - ACQuire:STOPAfter {RUNSTop|SEQuence}
        - ACQuire:STOPAfter?
        ```

    Info:
        - ``RUNSTop`` specifies that the instrument will continually acquire data, if
          ``ACQuire:STATE`` is turned on.
        - ``SEQuence`` specifies that the next acquisition will be a single-sequence acquisition.
    """


class AcquireState(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:STATE`` command.

    Description:
        - This command starts or stops acquisitions. When state is set to ON or RUN, a new
          acquisition will be started. If the last acquisition was a single acquisition sequence, a
          new single sequence acquisition will be started. If the last acquisition was continuous, a
          new continuous acquisition will be started. If RUN is issued in the middle of completing a
          single sequence acquisition (for example, averaging or enveloping), the acquisition
          sequence is restarted, and any accumulated data is discarded. Also, the instrument resets
          the number of acquisitions. If the RUN argument is issued while in continuous mode, a
          reset occurs and acquired data continues to acquire. If ``acquire:stopafter`` is SEQUENCE,
          this command leaves the instrument in single sequence, unlike the run/stop button which
          takes the instrument out of single sequence.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:STATE value`` command.

    SCPI Syntax:
        ```
        - ACQuire:STATE {<NR1>|OFF|ON|RUN|STOP}
        - ACQuire:STATE?
        ```

    Info:
        - ``<NR1> = 0`` stops acquisitions; any other value starts acquisitions.
        - ``OFF`` stops acquisitions.
        - ``ON`` starts acquisitions.
        - ``RUN`` starts acquisitions.
        - ``STOP`` stops acquisitions.
    """


class AcquireSequenceNumsequence(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:SEQuence:NUMSEQuence`` command.

    Description:
        - In single sequence acquisition mode, specify the number of acquisitions or measurements
          that comprise the sequence. The default is 1.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:SEQuence:NUMSEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence:NUMSEQuence?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:SEQuence:NUMSEQuence value``
          command.

    SCPI Syntax:
        ```
        - ACQuire:SEQuence:NUMSEQuence <NR1>
        - ACQuire:SEQuence:NUMSEQuence?
        ```

    Info:
        - ``<NR1>`` is the number of acquisitions or measurements that comprise the sequence.
    """


class AcquireSequenceMode(SCPICmdWrite):
    """The ``ACQuire:SEQuence:MODe`` command.

    Description:
        - In single sequence acquisition, the single sequence stop after count is based on the
          number of acquisitions.

    Usage:
        - Using the ``.write(value)`` method will send the ``ACQuire:SEQuence:MODe value`` command.

    SCPI Syntax:
        ```
        - ACQuire:SEQuence:MODe NUMACQs
        ```

    Info:
        - ``NUMACQs`` is the number of acquisitions.
    """


class AcquireSequenceCurrent(SCPICmdRead):
    """The ``ACQuire:SEQuence:CURrent`` command.

    Description:
        - In single sequence acquisition mode, this query returns the number of acquisitions or
          measurements in the sequence completed so far.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:SEQuence:CURrent?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence:CURrent?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ACQuire:SEQuence:CURrent?
        ```
    """


class AcquireSequence(SCPICmdRead):
    """The ``ACQuire:SEQuence`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:SEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.current``: The ``ACQuire:SEQuence:CURrent`` command.
        - ``.mode``: The ``ACQuire:SEQuence:MODe`` command.
        - ``.numsequence``: The ``ACQuire:SEQuence:NUMSEQuence`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._current = AcquireSequenceCurrent(device, f"{self._cmd_syntax}:CURrent")
        self._mode = AcquireSequenceMode(device, f"{self._cmd_syntax}:MODe")
        self._numsequence = AcquireSequenceNumsequence(device, f"{self._cmd_syntax}:NUMSEQuence")

    @property
    def current(self) -> AcquireSequenceCurrent:
        """Return the ``ACQuire:SEQuence:CURrent`` command.

        Description:
            - In single sequence acquisition mode, this query returns the number of acquisitions or
              measurements in the sequence completed so far.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:SEQuence:CURrent?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence:CURrent?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ACQuire:SEQuence:CURrent?
            ```
        """
        return self._current

    @property
    def mode(self) -> AcquireSequenceMode:
        """Return the ``ACQuire:SEQuence:MODe`` command.

        Description:
            - In single sequence acquisition, the single sequence stop after count is based on the
              number of acquisitions.

        Usage:
            - Using the ``.write(value)`` method will send the ``ACQuire:SEQuence:MODe value``
              command.

        SCPI Syntax:
            ```
            - ACQuire:SEQuence:MODe NUMACQs
            ```

        Info:
            - ``NUMACQs`` is the number of acquisitions.
        """
        return self._mode

    @property
    def numsequence(self) -> AcquireSequenceNumsequence:
        """Return the ``ACQuire:SEQuence:NUMSEQuence`` command.

        Description:
            - In single sequence acquisition mode, specify the number of acquisitions or
              measurements that comprise the sequence. The default is 1.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:SEQuence:NUMSEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence:NUMSEQuence?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``ACQuire:SEQuence:NUMSEQuence value`` command.

        SCPI Syntax:
            ```
            - ACQuire:SEQuence:NUMSEQuence <NR1>
            - ACQuire:SEQuence:NUMSEQuence?
            ```

        Info:
            - ``<NR1>`` is the number of acquisitions or measurements that comprise the sequence.
        """
        return self._numsequence


class AcquireNumframesacquired(SCPICmdRead):
    """The ``ACQuire:NUMFRAMESACQuired`` command.

    Description:
        - This query returns the number of FastFrame frames which have been acquired.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:NUMFRAMESACQuired?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMFRAMESACQuired?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ACQuire:NUMFRAMESACQuired?
        ```
    """


class AcquireNumavg(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:NUMAVg`` command.

    Description:
        - This command sets or queries the number of waveform acquisitions that make up an averaged
          waveform. Ranges from 2 to 10240.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:NUMAVg?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMAVg?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:NUMAVg value`` command.

    SCPI Syntax:
        ```
        - ACQuire:NUMAVg <NR1>
        - ACQuire:NUMAVg?
        ```

    Info:
        - ``<NR1>`` is the number of waveform acquisitions to average.
    """


class AcquireNumacq(SCPICmdRead):
    """The ``ACQuire:NUMACq`` command.

    Description:
        - This query-only command returns the number of waveform acquisitions that have occurred
          since the last time acquisitions were stopped.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:NUMACq?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:NUMACq?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ACQuire:NUMACq?
        ```
    """


class AcquireMode(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:MODe`` command.

    Description:
        - This command sets or queries the selected acquisition mode of the instrument.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:MODe value`` command.

    SCPI Syntax:
        ```
        - ACQuire:MODe {SAMple|PEAKdetect|AVErage|ENVelope}
        - ACQuire:MODe?
        ```

    Info:
        - ``SAMple`` specifies that the displayed data point value is the first sampled value that
          is taken during the acquisition interval. The CURVe? query, depending on sample rate, will
          result in either 8 bit or 16 bit data. In case of 8 bit data, the precision is also 8 bit.
          However, in case of 16 bit data, 12 bit precision data is zero padded in lower 4 bits.
          SAMple is the default mode.
        - ``PEAKdetect`` specifies the display of high-low range of the samples taken from a single
          waveform acquisition. The high-low range is displayed as a vertical column that extends
          from the highest to the lowest value sampled during the acquisition interval. PEAKdetect
          mode can reveal the presence of aliasing or narrow spikes.
        - ``AVErage`` specifies averaging mode, in which the resulting waveform shows an average of
          SAMple data points from several separate waveform acquisitions. The instrument processes
          the number of waveforms you specify into the acquired waveform, creating a running
          exponential average of the input signal. The number of waveform acquisitions that go into
          making up the average waveform is set or queried using the ``ACQuire:NUMAVg`` command.
        - ``ENVelope`` specifies envelope mode, where the resulting waveform displays the range of
          PEAKdetect from continued waveform acquisitions.
    """


class AcquireMaxsamplerate(SCPICmdRead):
    """The ``ACQuire:MAXSamplerate`` command.

    Description:
        - This query returns the maximum real-time sample rate.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:MAXSamplerate?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:MAXSamplerate?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ACQuire:MAXSamplerate?
        ```
    """


class AcquireFastaverageStopafter(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:FASTAVerage:STOPafter`` command.

    Description:
        - This command limits the total number of averages that will be accumulated. When the
          STOPAFTER count is achieved, acquisitions will stop. This setting must be a multiple of
          the LIMit count.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:STOPafter?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:STOPafter?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:FASTAVerage:STOPafter value``
          command.

    SCPI Syntax:
        ```
        - ACQuire:FASTAVerage:STOPafter <NR1>
        - ACQuire:FASTAVerage:STOPafter?
        ```

    Info:
        - ``<NR1>`` is the number of averages to accumulate. Must be between 1 and 1000000.
    """


class AcquireFastaverageState(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:FASTAVerage:STATE`` command.

    Description:
        - This command sets or returns the state of HW accelerated averaging. When fast averaging is
          enabled, batches of waveforms are accumulated in the hardware with 16-bit integer
          resolution. The accumulated data is uploaded to software, where additional averaging
          occurs in a floating-point math waveform. Each time data is uploaded from HW to SW, a
          programmable amount of dithering may be added to mitigate fine scale non-linearity in the
          ADC converter.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:STATE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:FASTAVerage:STATE value``
          command.

    SCPI Syntax:
        ```
        - ACQuire:FASTAVerage:STATE {ON|OFF|1|0}
        - ACQuire:FASTAVerage:STATE?
        ```

    Info:
        - ``0`` disables fast average mode.
        - ``1`` enables fast average mode. Any number value other than 0 turns this feature on.
        - ``OFF`` disables fast average mode.
        - ``ON`` enables fast average mode.
    """


class AcquireFastaverageLimit(SCPICmdWrite, SCPICmdRead):
    """The ``ACQuire:FASTAVerage:LIMit`` command.

    Description:
        - This command limits the number of averages performed in hardware at 16-bit resolution. The
          max value is 10240, but due to hardware limitations, it is usually best to use a value
          less than 256. This setting must be a divisor of the STOPafter count.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:LIMit?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:LIMit?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ACQuire:FASTAVerage:LIMit value``
          command.

    SCPI Syntax:
        ```
        - ACQuire:FASTAVerage:LIMit <NR1>
        - ACQuire:FASTAVerage:LIMit?
        ```

    Info:
        - ``<NR1>`` is the batch size to set. Must be between 1 and 10240.
    """


class AcquireFastaverage(SCPICmdRead):
    """The ``ACQuire:FASTAVerage`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.limit``: The ``ACQuire:FASTAVerage:LIMit`` command.
        - ``.state``: The ``ACQuire:FASTAVerage:STATE`` command.
        - ``.stopafter``: The ``ACQuire:FASTAVerage:STOPafter`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._limit = AcquireFastaverageLimit(device, f"{self._cmd_syntax}:LIMit")
        self._state = AcquireFastaverageState(device, f"{self._cmd_syntax}:STATE")
        self._stopafter = AcquireFastaverageStopafter(device, f"{self._cmd_syntax}:STOPafter")

    @property
    def limit(self) -> AcquireFastaverageLimit:
        """Return the ``ACQuire:FASTAVerage:LIMit`` command.

        Description:
            - This command limits the number of averages performed in hardware at 16-bit resolution.
              The max value is 10240, but due to hardware limitations, it is usually best to use a
              value less than 256. This setting must be a divisor of the STOPafter count.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:LIMit?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:LIMit?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:FASTAVerage:LIMit value``
              command.

        SCPI Syntax:
            ```
            - ACQuire:FASTAVerage:LIMit <NR1>
            - ACQuire:FASTAVerage:LIMit?
            ```

        Info:
            - ``<NR1>`` is the batch size to set. Must be between 1 and 10240.
        """
        return self._limit

    @property
    def state(self) -> AcquireFastaverageState:
        """Return the ``ACQuire:FASTAVerage:STATE`` command.

        Description:
            - This command sets or returns the state of HW accelerated averaging. When fast
              averaging is enabled, batches of waveforms are accumulated in the hardware with 16-bit
              integer resolution. The accumulated data is uploaded to software, where additional
              averaging occurs in a floating-point math waveform. Each time data is uploaded from HW
              to SW, a programmable amount of dithering may be added to mitigate fine scale
              non-linearity in the ADC converter.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:STATE?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:FASTAVerage:STATE value``
              command.

        SCPI Syntax:
            ```
            - ACQuire:FASTAVerage:STATE {ON|OFF|1|0}
            - ACQuire:FASTAVerage:STATE?
            ```

        Info:
            - ``0`` disables fast average mode.
            - ``1`` enables fast average mode. Any number value other than 0 turns this feature on.
            - ``OFF`` disables fast average mode.
            - ``ON`` enables fast average mode.
        """
        return self._state

    @property
    def stopafter(self) -> AcquireFastaverageStopafter:
        """Return the ``ACQuire:FASTAVerage:STOPafter`` command.

        Description:
            - This command limits the total number of averages that will be accumulated. When the
              STOPAFTER count is achieved, acquisitions will stop. This setting must be a multiple
              of the LIMit count.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage:STOPafter?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage:STOPafter?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``ACQuire:FASTAVerage:STOPafter value`` command.

        SCPI Syntax:
            ```
            - ACQuire:FASTAVerage:STOPafter <NR1>
            - ACQuire:FASTAVerage:STOPafter?
            ```

        Info:
            - ``<NR1>`` is the number of averages to accumulate. Must be between 1 and 1000000.
        """
        return self._stopafter


#  pylint: disable=too-many-instance-attributes
class Acquire(SCPICmdRead):
    """The ``ACQuire`` command.

    Description:
        - Queries the current acquisition state.

    Usage:
        - Using the ``.query()`` method will send the ``ACQuire?`` query.
        - Using the ``.verify(value)`` method will send the ``ACQuire?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ACQuire?
        ```

    Properties:
        - ``.fastaverage``: The ``ACQuire:FASTAVerage`` command tree.
        - ``.maxsamplerate``: The ``ACQuire:MAXSamplerate`` command.
        - ``.mode``: The ``ACQuire:MODe`` command.
        - ``.numacq``: The ``ACQuire:NUMACq`` command.
        - ``.numavg``: The ``ACQuire:NUMAVg`` command.
        - ``.numframesacquired``: The ``ACQuire:NUMFRAMESACQuired`` command.
        - ``.sequence``: The ``ACQuire:SEQuence`` command tree.
        - ``.state``: The ``ACQuire:STATE`` command.
        - ``.stopafter``: The ``ACQuire:STOPAfter`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "ACQuire") -> None:
        super().__init__(device, cmd_syntax)
        self._fastaverage = AcquireFastaverage(device, f"{self._cmd_syntax}:FASTAVerage")
        self._maxsamplerate = AcquireMaxsamplerate(device, f"{self._cmd_syntax}:MAXSamplerate")
        self._mode = AcquireMode(device, f"{self._cmd_syntax}:MODe")
        self._numacq = AcquireNumacq(device, f"{self._cmd_syntax}:NUMACq")
        self._numavg = AcquireNumavg(device, f"{self._cmd_syntax}:NUMAVg")
        self._numframesacquired = AcquireNumframesacquired(
            device, f"{self._cmd_syntax}:NUMFRAMESACQuired"
        )
        self._sequence = AcquireSequence(device, f"{self._cmd_syntax}:SEQuence")
        self._state = AcquireState(device, f"{self._cmd_syntax}:STATE")
        self._stopafter = AcquireStopafter(device, f"{self._cmd_syntax}:STOPAfter")

    @property
    def fastaverage(self) -> AcquireFastaverage:
        """Return the ``ACQuire:FASTAVerage`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:FASTAVerage?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:FASTAVerage?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.limit``: The ``ACQuire:FASTAVerage:LIMit`` command.
            - ``.state``: The ``ACQuire:FASTAVerage:STATE`` command.
            - ``.stopafter``: The ``ACQuire:FASTAVerage:STOPafter`` command.
        """
        return self._fastaverage

    @property
    def maxsamplerate(self) -> AcquireMaxsamplerate:
        """Return the ``ACQuire:MAXSamplerate`` command.

        Description:
            - This query returns the maximum real-time sample rate.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:MAXSamplerate?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:MAXSamplerate?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ACQuire:MAXSamplerate?
            ```
        """
        return self._maxsamplerate

    @property
    def mode(self) -> AcquireMode:
        """Return the ``ACQuire:MODe`` command.

        Description:
            - This command sets or queries the selected acquisition mode of the instrument.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:MODe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:MODe value`` command.

        SCPI Syntax:
            ```
            - ACQuire:MODe {SAMple|PEAKdetect|AVErage|ENVelope}
            - ACQuire:MODe?
            ```

        Info:
            - ``SAMple`` specifies that the displayed data point value is the first sampled value
              that is taken during the acquisition interval. The CURVe? query, depending on sample
              rate, will result in either 8 bit or 16 bit data. In case of 8 bit data, the precision
              is also 8 bit. However, in case of 16 bit data, 12 bit precision data is zero padded
              in lower 4 bits. SAMple is the default mode.
            - ``PEAKdetect`` specifies the display of high-low range of the samples taken from a
              single waveform acquisition. The high-low range is displayed as a vertical column that
              extends from the highest to the lowest value sampled during the acquisition interval.
              PEAKdetect mode can reveal the presence of aliasing or narrow spikes.
            - ``AVErage`` specifies averaging mode, in which the resulting waveform shows an average
              of SAMple data points from several separate waveform acquisitions. The instrument
              processes the number of waveforms you specify into the acquired waveform, creating a
              running exponential average of the input signal. The number of waveform acquisitions
              that go into making up the average waveform is set or queried using the
              ``ACQuire:NUMAVg`` command.
            - ``ENVelope`` specifies envelope mode, where the resulting waveform displays the range
              of PEAKdetect from continued waveform acquisitions.
        """
        return self._mode

    @property
    def numacq(self) -> AcquireNumacq:
        """Return the ``ACQuire:NUMACq`` command.

        Description:
            - This query-only command returns the number of waveform acquisitions that have occurred
              since the last time acquisitions were stopped.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:NUMACq?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMACq?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ACQuire:NUMACq?
            ```
        """
        return self._numacq

    @property
    def numavg(self) -> AcquireNumavg:
        """Return the ``ACQuire:NUMAVg`` command.

        Description:
            - This command sets or queries the number of waveform acquisitions that make up an
              averaged waveform. Ranges from 2 to 10240.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:NUMAVg?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMAVg?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:NUMAVg value`` command.

        SCPI Syntax:
            ```
            - ACQuire:NUMAVg <NR1>
            - ACQuire:NUMAVg?
            ```

        Info:
            - ``<NR1>`` is the number of waveform acquisitions to average.
        """
        return self._numavg

    @property
    def numframesacquired(self) -> AcquireNumframesacquired:
        """Return the ``ACQuire:NUMFRAMESACQuired`` command.

        Description:
            - This query returns the number of FastFrame frames which have been acquired.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:NUMFRAMESACQuired?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:NUMFRAMESACQuired?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ACQuire:NUMFRAMESACQuired?
            ```
        """
        return self._numframesacquired

    @property
    def sequence(self) -> AcquireSequence:
        """Return the ``ACQuire:SEQuence`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:SEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:SEQuence?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.current``: The ``ACQuire:SEQuence:CURrent`` command.
            - ``.mode``: The ``ACQuire:SEQuence:MODe`` command.
            - ``.numsequence``: The ``ACQuire:SEQuence:NUMSEQuence`` command.
        """
        return self._sequence

    @property
    def state(self) -> AcquireState:
        """Return the ``ACQuire:STATE`` command.

        Description:
            - This command starts or stops acquisitions. When state is set to ON or RUN, a new
              acquisition will be started. If the last acquisition was a single acquisition
              sequence, a new single sequence acquisition will be started. If the last acquisition
              was continuous, a new continuous acquisition will be started. If RUN is issued in the
              middle of completing a single sequence acquisition (for example, averaging or
              enveloping), the acquisition sequence is restarted, and any accumulated data is
              discarded. Also, the instrument resets the number of acquisitions. If the RUN argument
              is issued while in continuous mode, a reset occurs and acquired data continues to
              acquire. If ``acquire:stopafter`` is SEQUENCE, this command leaves the instrument in
              single sequence, unlike the run/stop button which takes the instrument out of single
              sequence.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:STATE?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:STATE value`` command.

        SCPI Syntax:
            ```
            - ACQuire:STATE {<NR1>|OFF|ON|RUN|STOP}
            - ACQuire:STATE?
            ```

        Info:
            - ``<NR1> = 0`` stops acquisitions; any other value starts acquisitions.
            - ``OFF`` stops acquisitions.
            - ``ON`` starts acquisitions.
            - ``RUN`` starts acquisitions.
            - ``STOP`` stops acquisitions.
        """
        return self._state

    @property
    def stopafter(self) -> AcquireStopafter:
        """Return the ``ACQuire:STOPAfter`` command.

        Description:
            - This command sets or queries whether the instrument continually acquires acquisitions
              or acquires a single sequence. Pressing SINGLE on the front panel button is equivalent
              to sending these commands: ``ACQUIRE:STOPAFTER SEQUENCE`` and ``ACQUIRE:STATE 1``.

        Usage:
            - Using the ``.query()`` method will send the ``ACQuire:STOPAfter?`` query.
            - Using the ``.verify(value)`` method will send the ``ACQuire:STOPAfter?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ACQuire:STOPAfter value`` command.

        SCPI Syntax:
            ```
            - ACQuire:STOPAfter {RUNSTop|SEQuence}
            - ACQuire:STOPAfter?
            ```

        Info:
            - ``RUNSTop`` specifies that the instrument will continually acquire data, if
              ``ACQuire:STATE`` is turned on.
            - ``SEQuence`` specifies that the next acquisition will be a single-sequence
              acquisition.
        """
        return self._stopafter
